#ifndef POTENTIALS_H
#define POTENTIALS_H

#include "precision.hpp"
#include "vec3.h"

/**
 * Result struct for the Pair Potential
 */
struct ForceAndEnergy {
  real energy;
  Vec3<real> force;

  inline static ForceAndEnergy zero() { return {0.0, {0.0, 0.0, 0.0}}; };
};

/**
 * Abstract implementation of a Pair Potential.
 * Pair potentials are potentials which depend solely on the distance
 * between two particles. These do not include multi-body potentials such as
 * EAM
 *
 */
struct PairPotential {
  real m_rcutoffsq;

  PairPotential(real rcutoff) : m_rcutoffsq(rcutoff * rcutoff) {};
  virtual ~PairPotential() = 0;

  /**
   * Calculate the force and energy for a specific atom pair based on a
   * displacement vector r.
   */
  virtual ForceAndEnergy calc_force_and_energy(Vec3<real> r) = 0;
};

struct LennardJones : PairPotential {
  real m_epsilon;
  real m_sigma;

  LennardJones(real sigma, real epsilon, real rcutoff)
      : PairPotential(rcutoff), m_epsilon(epsilon), m_sigma(sigma) {};

  ForceAndEnergy calc_force_and_energy(Vec3<real> r);

  ~LennardJones() {};
};

#endif
